%SELECT_CALIB_POINTS Lets a user select calibration points from scene
%
%The function asks user to enter number of reference points available, than
%cycles through select-file-choose-point-enter-coordinates cycle
%
%PARAMETERS:        video_suffix = string containing desired video file
%                                   suffix that will be visible in the
%                                   choose file dialog, * is the default
%                                   value
%RETURNS:           calibration_pairs = [x1 y1 u1 v1 ; ...]
%                                   x,y = world coordinates [m]
%                                   u,u = picture coordinates

function [functionDone calibration_pairs ] = select_calib_points(video_suffix)


    functionDone = true;

    if nargin < 1; video_suffix = '*'; end
    
    stop =false;
    while (stop == false)
        pointsNum = input('How many calibration points can you provide?: ');

        stop = true;
        if((pointsNum - round(pointsNum)) ~= 0)
            disp('Must be an integer, aborting.');
            stop =false;
        elseif (pointsNum < 4) 
            disp('Must be 4 or more, aborting.');
            stop =false;
        end
    end   
           
    calibration_pairs = zeros(pointsNum, 4);

    for index = 1 : 1 : pointsNum
    
   
        disp('-----------------------------------------------------------')
        disp('Chose video which you want to calibrate')
        
        [File,Path]=uigetfile(strcat('*.',video_suffix),...
        'Select the video file');        
    
        if (File == 0) & (Path == 0)
            disp('No file selected'); 
            functionDone = false;
            break
        end
    
        fname = strcat(Path,File);

        disp('video loading...')
        video_stream = createVideoReader(fname);
    
        image = read(video_stream, round(video_stream.NumberOfFrames/2));
    
        figure(10);
        imshow(image);
        text(20, 20, sprintf('Choose point #%5d',index), 'BackgroundColor', 'white');
        [refU refV] = ginput(1);
        close(figure(10));
    
        refX = input('What is the X coordinate [m] of this point?: ');
        refY = input('What is the Y coordinate [m] of this point?: ');
    
        
        calibration_pairs(index,:) = [refX refY refU refV];
        
    
    end
    
end

