%##########################################################################
%
%
% author : Premysl Kafka (kafka.premyls@gmail.com)
%
%##########################################################################

function [] = run_the_tracker ()
clc
    disp(' ')
    disp('#--------------------------------------------------------------')
    disp('# Program was launched')
    disp('#--------------------------------------------------------------')
    add_paths();
    
     savePath = uigetdir(pwd(),'Select file which will represent MEASUREMENT'); % TODO koukni co misto toh MEASUREMENT

    if(savePath ~= 0)
        disp(['You have chosen: ' savePath])
        
        LOAD = 'load';
        SET = 'create';
        END = 'end';

        an = questdlg('Program was launched. Do you wath to load or create kalibrations points.', 'kalibration', ...
            LOAD,SET,END,LOAD);
    
        trackingRes.track = NaN;
        calibPoints =NaN;
    
        % CALIBRATION
        if( strcmp(an, LOAD) )       
            calibPoints = load_calibration();            
        elseif ( strcmp(an, SET) )        
            calibPoints = create_calibration(savePath);    
        end   

        % TRACKER
        if(isnan(calibPoints) == false)
           trackingRes = run_track(savePath);

    %        trackingRes.track

            % HOMOGRAFY
            if(isnan(trackingRes.track) == false)
                track_txy=run_homografy(calibPoints, trackingRes); 
                 % SYNCHRONIZATION
                 pause(2)
                run_synchronization(track_txy, savePath);
            end
        end
    end
    
    disp(' ')
    disp('Program was ended correctly.')
    disp(' ')
end  %end of funtion run


%#############################################################################################
function[calibPoints]  = load_calibration()
    
    [filename, pathname] = uigetfile( {'*.mat','MAT-files (*.mat)'},'Load file with calibration points');
    
    if( filename ~= 0)
        path = strcat(pathname,filename);

        loaded = load(path);
        calibPoints=loaded.calibPoints;
        disp(['Calibration points were loaded: ' path])
    else
        calibPoints =NaN;
    end

end  %end of function load_calibration



%#############################################################################################
function [calibPoints] = create_calibration(savePath)
    
    [funcDone calibPoints] = select_calib_points('AVI');
%     calibPoints
    if( funcDone == true )
        [filename, pathname] = uiputfile('.mat', 'Save calibration points' ,savePath);
        if( (filename ~=0 ))
            path = strcat(pathname,filename);
            save(path, 'calibPoints');
            disp(['calibration points were saved: ' path])
        else
            calibPoints =NaN;
        end
    else
       calibPoints =NaN; 
    end
    
end



%#############################################################################################
%################################ TRACKING METHODS############################################
%#############################################################################################

function [trackingRes] = run_track( savePath)

	disp(' ')
    disp(' ')
    disp('#--------------------------------------------------------------')
    disp('# TRACKING')
    disp('#--------------------------------------------------------------')
    
    trackingRes.track =NaN;
    
    LOAD = 'load';
    SET = 'create';

    an = questdlg('Do you want to load or create tracking?', 'tracking', ...
        LOAD,SET,'Cancel',LOAD);
    
    if( strcmp(an, LOAD) )       
           trackingRes = load_tracking();   
    elseif ( strcmp(an, SET) )        
           trackingRes = create_tracking(savePath);
    end   
end

%#############################################################################################
function[trackingRes]  = load_tracking()
    trackingRes.track = NaN;
    
    stop=false;
    while (stop == false)
        [filename, pathname] = uigetfile( {'*.mat','MAT-files (*.mat)'},'Load the file wiht tracking');

        if(filename ~= 0)
            path = strcat(pathname,filename);

            stop=true;
            loaded = load(path);
            try
                trackingRes=loaded.trackingRes;
            catch e
                stop=false;
                disp('You have to load structure with track. Try it again.')
            end
            if (stop == true)
                disp(['You loaded file with tracking: ' path] ) 
            end
        else
            %end of loop
            break
        end
    end
        

end  %end of function load_calibration



%#############################################################################################
function [trackingRes] = create_tracking(savePath)

    WITHOUT = '2D';
    WITH = '2D with azimuth';
    
    an = questdlg(' Chose between 2D or 2D with azimuth computing.','Chose tracking mode',WITHOUT, WITH, WITHOUT);
    
    [filename, pathname] = uigetfile( {'*.AVI','VIDEO-files (*.AVI)'; '*.*',  'All Files (*.*)'},'Load a video to be tracked');
    if(filename ~= 0)
        path = strcat(pathname,filename);

        if( strcmp(an, WITH) )       
            [track_temp frames I_temp] = trackRobot(path,2);
            trackingRes.track = [frames' track_temp{1}'];
            trackingRes.track2 = [frames' track_temp{2}'];
            trackingRes.image = I_temp;
            
        elseif ( strcmp(an, WITHOUT) )        
            [track_temp frames I_temp] = trackRobot(path,1);
            trackingRes.track = [frames' track_temp{1}'];
            trackingRes.image = I_temp;
            trackingRes.track2 = NaN;
        end 

        [filename, pathname] = uiputfile('*.mat','Save the tracking file', savePath);
        path = strcat(pathname,filename);
        disp(['Tracking file was saved: ' path])
        save(path, 'trackingRes');
    
    else
        trackingRes.track = NaN;
    end

end




%#############################################################################################
%################################ HOMOGRAFY METHODS ##########################################
%#############################################################################################



function [track_txy] = run_homografy(xyuv, trackingRes)
    disp(' ')
    disp(' ')
    disp('#--------------------------------------------------------------')
    disp('# HOMOGRAFY')
    disp('#--------------------------------------------------------------')
    
    if (isnan(trackingRes.track2))
        track_txy = homografy_main(xyuv, trackingRes, true);   
    else
        trackingResTemp.track = trackingRes.track;
        trackingResTemp.image = trackingRes.image;
        track1_txy = homografy_main(xyuv, trackingResTemp, false);
        
        trackingResTemp.track = trackingRes.track2;
        track2_txy = homografy_main(xyuv, trackingResTemp, false);
        
        track_txy(:,1:3) = [ track1_txy(:,1) (track1_txy(:,2:3)+track2_txy(:,2:3))*0.5 ];
        track_txy(:,4) = -atan2(track1_txy(:,3)-track2_txy(:,3),track1_txy(:,2)-track2_txy(:,2))*180/pi;
        
        
        
        plot_homography(track_txy,trackingRes);
        
        
        
    end   
        
    disp('Homografy is complete.')
end

%#############################################################################################
%################################ SYNCHRONIE METHODS #########################################
%#############################################################################################



function [] = run_synchronization(track_txy, savePath)
    disp(' ')
    disp(' ')
    disp('#--------------------------------------------------------------')
    disp('# SYNCHRONIZATION')
    disp('#--------------------------------------------------------------')
    disp('Select a _vpa_ log file from INSO')

    % get the vpa file 
    [filename, infilepath] = uigetfile('*.csv','Select a _vpa_ log file from INSO');
    
    if (filename ~= 0)         

        vpa_data = importdata(strcat(infilepath,filename));
        [track_interpolated_cropped vpa_cropped] = synch_vpa_and_track(track_txy, vpa_data);
        
        % VK: plot result in XY
        
        figure();
        hold on;
        plot(-track_interpolated_cropped(:,3) + track_interpolated_cropped(1,3)*ones(size(track_interpolated_cropped,1),1),...
            track_interpolated_cropped(:,2) - track_interpolated_cropped(1,2) * ones(size(track_interpolated_cropped,1),1),'b');
        plot(vpa_cropped(:,9) ,vpa_cropped(:,8),'r');
        grid on;
        axis equal;
        title('Top view');
        xlabel('X coordinate [m]');
        ylabel('Y coordinate [m]');
        legend('Tracked','INSO');
        
        % end VK
        
        filename = modify_filename(filename);
        savePath=strcat(savePath, get_file_separator() );    
        path=strcat(savePath, filename );      
        save_cvs_file( track_interpolated_cropped, path);
    end 
end

%#############################################################################################
% This function save matrix as a cvs file. If path is wrong. User have to 
% chose another right path with save dialog box.
%
function [] = save_cvs_file(matrix, path)

    SEPARATOR = ', ';
    
    while (true)
        fileID = fopen(path,'w');

        if (fileID > -1)

            J_SIZE = size(matrix,2);
            for i=1:size(matrix,1)
               for j=1:J_SIZE
                   fprintf(fileID,'%f',matrix(i,j));   

                   if(j < J_SIZE)
                        fprintf(fileID,'%s',SEPARATOR);  
                   end
               end  
               fprintf(fileID, '\n');
            end
            fclose(fileID);
            disp(['File was saved: ' path ])
            %end of loop
            break
        else
            errMessage = strcat(path,' File can not be saved. Chose another path','replace');
            disp(errMessage)
%             errordlg(errMessage,'File Error');
            [filename, pathname] = uiputfile('*.csv','Chose another path');
            
            if isequal(filename,0) || isequal(pathname,0)
               disp('Canceled')
               %end of loop
               break
            else
               path =strcat(pathname, filename);
               disp(['User selected',path])
            end
            
        end
    end

end % end of function




%#############################################################################################
%################################ OTHER METHODS #############################################
%#############################################################################################

%#############################################################################################
function [] = add_paths()
    
    if  strcmp( get_os_type(), 'linux')
        add_paths_on_linux()
    elseif  strcmp( get_os_type(), 'windows')
        add_paths_on_windows()
    end
end

%#############################################################################################
function [] = add_paths_on_linux()
    disp('You are using Linux/Unix OS')
    root = pwd();
    path=strcat(root,'/vyber_ref_bodu');
    addpath(path)
    
    path=strcat(root,'/trackery/histogram_tracker');
    addpath(path)
    
    path=strcat(root,'/transformace/vlada_k_homografie');
    addpath(path)
    
    path=strcat(root,'/synchronizace');
    addpath(path)
end

%#############################################################################################
function [] = add_paths_on_windows()
    disp('You are using Windows OS')
    root = pwd();
    path=strcat(root,'\vyber_ref_bodu');
    addpath(path)
    
    path=strcat(root,'\trackery\histogram_tracker');
    addpath(path)
    
    path=strcat(root,'\transformace\vlada_k_homografie');
    addpath(path)
    
    path=strcat(root,'\synchronizace');
    addpath(path)
end

%#############################################################################################
% This function adds suffix '.mat' to save path. If
% input path contains this suffix, nothing will be
% added.
function [path] = add_mat_suffix(path)
    sufPosition = findstr(path, '.mat');
%     sufPosition
    
    add = true;
    for i=1:length(sufPosition)
       if (length(path)-3) == sufPosition(1,i);
           add = false;
           break
       end
    end
    
    if add == true
        path = strcat(path,'.mat');
    end

end   %end of method: add_mat_suffix


%#############################################################################################
function [filename] = modify_filename(filename)
    filename = strrep(filename, '_vpa_', '_track_');
end

%#############################################################################################
function [res] = get_os_type()
     res = 'linux';
    os = lower(system_dependent('getos'));
    
    num = findstr(os, 'windows'  );
    if  isempty(num) == false
        res = 'windows';
    end
    
end % end of function

%#############################################################################################
function [fileSeparator] = get_file_separator()
    if strcmp(get_os_type() ,'linux')
        fileSeparator = '/';
    end
    
    if strcmp(get_os_type() ,'windows')
        fileSeparator = '\';
    end
end
