function [H] = eval_homography(xyuv)
%EVAL_HOMOGRAPHY Evaluates homography matrix H using SVD
%  xyuv are corresponding points for the xy = H*uv equation. There must be
%  at least 4 pairs : [ x1 y1 u1 v1; x2 y2 u2 v2; ...]

if nargin ~= 1 ;
    error('Wrong number of parameters, one parameter expected, see documentation!');
end

if (size(xyuv,2) ~= 4) || (size(xyuv,1) < 4) ;  
    error('The input matrix dimensions must be (n,4) where n >= 4');
end



n = size(xyuv,1); % number of point correspondences
 
A = zeros(2*n,9);
 
 for i = 1:n
 
     xy_h = [xyuv(i,1:2) 1]'; % world point in homogenous coord.
     uv_h = [xyuv(i,3:4) 1]'; % image coordinates in homogenous coord.
     
     A_i = [0 0 0 -xy_h(3)*(uv_h') xy_h(2)*(uv_h') ; ...
            xy_h(3)*uv_h' 0 0 0 -xy_h(1)*(uv_h')];
            
     A((2*i-1):(2*i),:) = A_i;
 
 end
 
[~,~,V] = svd(A,0);
h = V(:,end);

H = [h(1:3)';h(4:6)';h(7:9)'];

end

