classdef HistogramTracker < handle
    %#ok<*PROP>
    
    properties
        H = ones(256, 256, 3);
        B;
        Brect;
        x = 0;
        y = 0;
        r = 1;
        rMargin = 2;
        cmask;
        tHoldL = 0;
        tHoldU = 1;
        rect;
    end

    methods
        function obj = HistogramTracker(frame, mask)
            [obj.x, obj.y] = centerOfMass(mask);
            obj.r = maskRadius(mask, obj.x, obj.y);
            obj.r = obj.r * obj.rMargin;

            mask = repmat(mask, [1 1 3]);
            samples = frame(mask);

            obj.H = createHSVmodel(samples, 0.005*[1 1 1], [0.001 0.02 0.02]);
            
            obj.cmask = circleMask(round(2*obj.r), round(2*obj.r), obj.r, obj.r, obj.r);
        end
        
        function [x, y] = trackFrame(obj, frame, fullB)
            if nargin < 3; fullB = false; end;

            rect = [floor(obj.x-obj.r), floor(obj.y-obj.r), ceil(2*obj.r), ceil(2*obj.r)];
            
            if ~fullB; frame = imcrop(frame, rect); end

            B = backProjection(uint8(rgb2hsv(frame)*255), obj.H);
            B = prod(B,3);

            %se = strel('disk', 2);
            %B = imopen(B,se);
            %B = medfilt2(B);
            
            % Adaptive treshold
            % Method 1
            %cs = (smooth(diff(cumsum(hist(B(:),256))), 10));
            %obj.tHoldL = (find(cs<max(cs)/50, 1, 'first') - 1) / 255;
            % Method 2
            p = 0.5;    % Percentual range of lower values which are cut off
            obj.tHoldL = min(min(B)) * (1 - p) + max(max(B)) * p;
            
            
            if obj.tHoldL > 0; B(B<obj.tHoldL) = 0; end
            if obj.tHoldU < 1; B(B>obj.tHoldU) = 1; end

            obj.B = B;
            
            if fullB; B = imcrop(B, rect); end
            
            
            [nx,ny] = centerOfMass(B, obj.cmask);
            x = nx + rect(1);
            y = ny + rect(2);

            % Adaptive radius of the circle window
            %obj.r = radiusOfMass(B, 0.8, x-rect(1), y-rect(2), obj.cmask) * 1.5 + 5;
            %obj.cmask = circleMask(round(2*obj.r), round(2*obj.r), obj.r, obj.r, obj.r);
            
            obj.x = x;
            obj.y = y;
            obj.rect = rect + [0 0 1 1];
            
            obj.Brect = obj.rect;
            if fullB; obj.Brect = [0 0 size(frame,2) size(frame,1)]; end
        end
        
        function [] = setTreshold(obj, lower, upper)
            obj.tHoldU = upper;
            obj.tHoldL = lower;
        end
    end
end
