classdef TrackerVisualization < handle
    %UNTITLED Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (Access = protected)
        fig
        ax1
        ax2
        an1
        imgA
        imgB
        crosses;
        circles;
    end
    
    methods
        function obj = TrackerVisualization
            obj.fig = figure('Position',[0 0 2*480+3*20 360+2*20], ...
                'Resize','off');
            movegui(obj.fig,'center');
    
            obj.ax1 = axes('Units','pixels','Position',[20 20 480 360]);
            obj.ax2 = axes('Units','pixels','Position',[480+2*20 20 480 360]);            
            set([obj.ax1 obj.ax2], 'XTick',[],'YTick',[],'Color',[0 0 0],'Visible','on');
            
            obj.an1 = annotation('textbox','String','firstline \n second line', ...
                'Units','pixels','Interpreter','none','FitBoxToText','on', ...
                'BackgroundColor','white','LineStyle','none');
            obj.setAnnotText('...');
        end
        
        function setImgA(obj, img)
            if isempty(obj.imgA)
                obj.imgA = imshow(img,'Parent',obj.ax1);
            else
                set(obj.imgA,'CData',img);
            end
        end
        
        function setImgB(obj, img, rect)
            if isempty(obj.imgB)
                obj.imgB = imshow(img,'Parent',obj.ax2);
                set(obj.ax2,'XTick',[],'YTick',[],'Color',[0 0 0],'Visible','on');
                %axis(obj.ax2,'equal');
            end
            
            s = size(img);
            ws = [480, 360] + 0.5;
            scale = max([s(2), s(1)] ./ ws);
            pad = (ws*scale - [s(2), s(1)]) / 2;
            
            if nargin < 3; rect = [0 0 0 0]; end
            
            set(obj.imgB,'CData',img,'XData',[rect(1)+1, rect(1)+s(2)],'YData',[rect(2)+1, rect(2)+s(1)]);
            xl = [-pad(1), pad(1)+s(2)] + 0.5 + rect(1);
            yl = [-pad(2), pad(2)+s(1)] + 0.5 + rect(2);
            set(obj.ax2,'XLim',xl,'YLim',yl);
        end
        
        function setCross(obj, no, coord, color)
            if obj.cellInvalidHandle(obj.crosses, no);
                if nargin < 4; color = [0 0 0]; end
                obj.crosses{no} = line(coord(1),coord(2), ...
                    'Parent',obj.ax1,'Marker','+','MarkerSize',50, ...
                    'Color',color,'LineWidth',1);
            else
                set(obj.crosses{no},'XData',coord(1),'YData',coord(2));
            end
        end
        
        function setCircle(obj, no, pos, color)
            if obj.cellInvalidHandle(obj.circles, no);
                if nargin < 4; color = [0 0 0]; end
                obj.circles{no} = rectangle('Parent',obj.ax2, ...
                    'EdgeColor',color,'Position',pos,'Curvature',[1,1]);
            else
                set(obj.circles{no},'Position',pos);
            end
        end
        
        function setAnnotText(obj, text)
            if ~iscell(text); text = regexp(text,'\n','split'); end
            
            set(obj.an1,'String',text);
            p = get(obj.an1,'Position');
            p(1) = 25;
            p(2) = 400 - 25 - p(4);
            set(obj.an1,'Position',p);
        end
        
        function F = getFrame(obj)
            F = getframe(obj.fig);
        end
        
        function delete(obj)
            if ishandle(obj.fig)
                close(obj.fig);
            end
        end
    end
    
    methods (Static, Access = protected)
        function b = cellInvalidHandle(cellvar, index)
            b = index > length(cellvar) || ...
                isempty(cellvar{index}) || ...
                ~ishandle(cellvar{index});
        end
    end
    
end

