function [H] = createHSVmodel(samples, RGBsigma, HSVsigma)
    N = 100000;
    nu = numel(samples);
    
    if isinteger(samples)
        samples = double(samples) / 255;
    end
    
    samples = reshape(samples, nu/3, 3);
    samples = repmat(samples, ceil(N/nu*3), 1);
        
    R = mvnrnd(0*[1 1 1], diag(RGBsigma), size(samples,1));
    samples2 = samples + R;
    samples2 = samples2(min(samples2,[],2) >= -0.5/255, :);
    samples2 = samples2(max(samples2,[],2) <= 255.5/255, :);
    samples2 = rgb2hsv(samples2);
    samples2 = round(samples2 * 255);
    H2 = histc(samples2, (0:255));
    H2 = bsxfun(@(x,y) x./y, H2, max(H2));
    
    samples = rgb2hsv(samples);
    R = mvnrnd(0*[1 1 1], diag(HSVsigma), size(samples,1));
    samples = samples + R;
    samples = samples(min(samples(:,2:3),[],2) >= -0.5/255, :);
    samples = samples(max(samples(:,2:3),[],2) <= 255.5/255, :);
    samples = round(samples * 255);
    
    while true
        m = samples(:,1) > 255;
        if ~any(m); break; end
        samples(m,1) = samples(m,1) - 256;
    end
    
    while true
        m = samples(:,1) < 0;
        if ~any(m); break; end
        samples(m,1) = samples(m,1) + 256;
    end
    
    H = histc(samples, (0:255));
    H = bsxfun(@(x,y) x./y, H, max(H));
    
    H = arrayfun(@max, H, H2);
    
    for i = (1:3); H(:,i) = smooth(H(:,i), 20); end
end
