function [coords, framesNo, I_out] = trackRobot(movieFile, objsNo)
    if nargin < 2; objsNo = 1; end

    fprintf('Opening the movie file...\n');
    vr = createVideoReader(movieFile);
    
    showFullBackprojection = false;

    scale = 1;
    
    firstFrame = 10;
    lastFrame = inf;
    lastFrame = min(lastFrame, vr.NumberOfFrames);
    
    trackers = cell(1,objsNo);
    coords = cell(1,objsNo);
    colors = cell(1,objsNo);
    
    %% Input of objects to be tracked
    I = read(vr, firstFrame);
    I = imresize(I, scale);
    I_out = I;

    imshow(I);
    
    for i = 1:objsNo
        fprintf('Select %d. object to be tracked...\n', i);
        
        % Choose appropriate method for a region selection
        h = imellipse;
        %h = impoly;
        
        wait(h);
        m = h.createMask;
        h.delete;

        trackers{i} = HistogramTracker(I, m);
        coords{i} = NaN(2, lastFrame-firstFrame+1);
       
        [~, hue] = max(trackers{i}.H(:,1));
        colors{i} = hsv2rgb([(hue-1)/255 1 1]);
    end
    
    close;
    
    % Show histograms created by trackers
    for i = 1:objsNo
        figure; stairs(trackers{i}.H);
    end
    
    
    vis = TrackerVisualization();
    
    % Maybe colors taken from video frame are badly visible
    %colors = [1 0 0; 0 1 0; 0 0 1];
    
    %% Track all objects in all video frames
    for i = (1:1:lastFrame-firstFrame+1)
        frame = read(vr, i+firstFrame-1);
        frame = imresize(frame, scale);
        
        % Tracking
        for j = 1:objsNo
            [x, y] = trackers{j}.trackFrame(frame, showFullBackprojection);
            coords{j}(:,i) = [x; y];
        end
        
        % Its possible to speed up the process by exclusion of UI repaint
        %if mod(i-1, round(vr.FrameRate)) ~= 0; continue; end
        
        % Visualization
        vis.setImgA(frame);
        vis.setAnnotText(sprintf('frame: %05d;  position{1}: [%3.1f; %3.1f]', ...
            i, coords{1}(1,i), coords{1}(2,i)));

        collage = MaskCollageBuilder();
        for j = 1:objsNo
            c = colors{j};
            vis.setCross(j, coords{j}(:,i), c);
            vis.setCircle(j, trackers{j}.rect, c);
            collage.addMask(trackers{j}.B, trackers{j}.Brect, c);
        end
        
        [I, Irect] = collage.getCollage();
        vis.setImgB(I, Irect);
        
        drawnow();
    end
    
    framesNo = (firstFrame:lastFrame);
    
    vr.delete();
end

