function [] = movieToImages(movieFile, outputFormat, numberOfFrames, fps)
% MOVIETOIMAGES - converts movie file to a serie of images. Images are 
% saved in a newly created directory named after the movieFile. All 
% existing images are deleted.
%
%   PARAMETERS:
%       movieFile - movie input file's name
%       ouputFormat - defaults to 'jpg'
%       numberOfFrames - number of frames being extracted. Defaults to inf
%           (all frames).
%       fps - number of frames exported per second of movie
%
%   EXAMPLES:
%       movieToImages('MVI_8202.avi', 'jpg', inf, 1)
%

if nargin < 1; movieFile = 'MVI_8226.AVI'; end
if nargin < 2; outputFormat = 'jpg'; end
if nargin < 3; numberOfFrames = inf; end

vr = VideoReader(movieFile);

if nargin < 4; fps = vr.FrameRate; end


[path, fileName] = fileparts(movieFile);
framesDir = fullfile(path, fileName);

if exist(framesDir, 'dir');
    delete(fullfile(framesDir, '*'));
else
    mkdir(framesDir);
end

if vr.NumberOfFrames < numberOfFrames
    numberOfFrames = vr.NumberOfFrames;
end

df = vr.FrameRate / fps;
for i = (1:df:numberOfFrames)
    i = floor(i);
    frame = read(vr, i);
    
    %frame = frame(:,:,1);

    frameName = sprintf('%05d.%s', i-1, outputFormat);
    imwrite(frame, fullfile(framesDir, frameName), outputFormat);
end
