classdef Transform3D < handle
    %UNTITLED4 Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        T = eye(4);
        Tcell = cell(0);
        Jcell = cell(0);
    end
    
    methods
        function tranMat(obj, T2, J2)
            obj.T = T2 * obj.T;
            obj.Tcell = {obj.Tcell{:}, T2};
            obj.Jcell = {obj.Jcell{:}, J2};
        end
        
        function rotX(obj, alpha)
            a = alpha;
            T2 = [1,0,0,0; 0,cos(a),-sin(a),0; 0,sin(a),cos(a),0; 0,0,0,1];
            J2 = [0,0,0,0; 0,-sin(a),-cos(a),0; 0,cos(a),-sin(a),0; 0,0,0,0];
            obj.tranMat(T2, J2);
        end
        
        function rotZ(obj, alpha)
            a = alpha;
            T2 = [cos(a),-sin(a),0,0; sin(a),cos(a),0,0; 0,0,1,0; 0,0,0,1];
            J2 = [-sin(a),-cos(a),0,0; cos(a),-sin(a),0,0; 0,0,0,0; 0,0,0,0];
            obj.tranMat(T2, J2);
        end
        
        function moveCoord(obj, coord, d)
            T2 = eye(4);
            T2c = [T2(coord,1:3), d];
            T2 = [T2(1:coord-1,:); T2c; T2(coord+1:end,:)];
            J2 = zeros(4); J2(coord,4) = 1;
            obj.tranMat(T2, J2);
        end
        
        function moveX(obj, dx)
            obj.moveCoord(1, dx);
        end
        
        function moveY(obj, dy)
            obj.moveCoord(2, dy);
        end
        
        function moveZ(obj, dz)
            obj.moveCoord(3, dz);
        end
        
        function translation(obj, t)
            obj.moveX(t(1));
            obj.moveY(t(2));
            obj.moveZ(t(3));
        end
        
        function eulerAngles(obj, alpha, beta, gama)
            obj.rotZ(alpha);
            obj.rotX(beta);
            obj.rotZ(gama);
        end
        
        function scale(obj, s)
            T2 = [s 0 0 0; 0 s 0 0; 0 0 s 0; 0 0 0 1];
            J2 = [1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 0];
            obj.tranMat(T2, J2);
        end
        
        function scaleXY(obj, s)
            T2 = [s 0 0 0; 0 s 0 0; 0 0 1 0; 0 0 0 1];
            J2 = [1 0 0 0; 0 1 0 0; 0 0 0 0; 0 0 0 0];
            obj.tranMat(T2, J2);
        end
        
        function [T] = getTransformation(obj)
            T = obj.T;
        end
        
        function [J] = getJacobian(obj, x)
            x = x(:);
            k = length(obj.Tcell);
            J = zeros(4, k);
            for i = (1:length(obj.Tcell))
                J(:,1:i-1) = obj.Tcell{i} * J(:,1:i-1);
                J(:,i) = obj.Jcell{i} * x;
                x = obj.Tcell{i} * x;
            end
        end
    end
    
    methods (Static)
        function [T] = transform(p)
            T = Transform3D;
            T.translation(p(1:3));
            T.eulerAngles(p(4), p(5), p(6));
            T.scaleXY(p(7));
        end
        
        function test()
            ps = sym('ps',[1 7]);
            p = 10*rand(1,7);
            X = [rand(3,1); 1];
            
            T = Transform3D.transform(ps);
            J1 = jacobian(T.T*X, [ps]);
            J1 = subs(J1, ps, p);
            
            T = Transform3D.transform(p);
            J2 = T.getJacobian(X);

            assert(all(all(abs(J1-J2) < 1e-10)));
        end
    end
    
end

